"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Differ = void 0;
class Differ {
    static #type() {
        return {
            string: (value) => value?.constructor?.name === 'String',
            regex: (value) => value?.constructor?.name === 'RegExp',
            symbol: (value) => value?.constructor?.name === 'Symbol',
            function: (value) => value?.constructor?.name === 'Function',
            bigint: (value) => value?.constructor?.name === 'BigInt',
            number: (value) => value?.constructor?.name === 'Number' && !Number.isNaN(value),
            nan: (value) => Number.isNaN(value),
            null: (value) => value === null,
            undefined: (value) => value === undefined,
            boolean: (value) => value?.constructor?.name === 'Boolean',
            array: (value) => Array.isArray(value),
            date: (value) => value?.constructor?.name === 'Date' && !Number.isNaN(value.getTime()),
            set: (value) => value?.constructor?.name === 'Set',
            object: (value) => value?.constructor?.name === 'Object',
            map: (value) => value?.constructor?.name === 'Map'
        };
    }
    static equal(v1, v2) {
        const [val1, val2] = [v1, v2].map((v) => {
            if (Differ.#type().map(v))
                return Object.fromEntries(...val1.entries());
            if (Differ.#type().set(v))
                return [...v];
            return v;
        });
        if (val1 === val2)
            return true;
        if (Differ.#type().string(val1))
            return val1 === val2;
        if (Differ.#type().number(val1))
            return val1 === val2;
        if (Differ.#type().bigint(val1))
            return val1 === val2;
        if (Differ.#type().boolean(val1))
            return val1 === val2;
        if (Differ.#type().null(val1))
            return val1 === val2;
        if (Differ.#type().undefined(val1))
            return val1 === val2;
        if (Differ.#type().function(val1))
            return val1 === val2;
        if (Differ.#type().regex(val1))
            return Differ.#type().regex(val2) && val1.source === val2.source;
        if (Differ.#type().nan(val1))
            return Differ.#type().nan(val2);
        if (Differ.#type().date(val1))
            return Differ.#type().date(val2) && val1.getTime() === val2.getTime();
        if (Differ.#type().array(val1)) {
            if (!Differ.#type().array(val2))
                return false;
            if (val1.length !== val2.length)
                return false;
            return val1.every((c, i) => Differ.equal(c, val2.at(i)));
        }
        const keys1 = Object.keys(val1 ?? {});
        const keys2 = Object.keys(val2 ?? {});
        if (keys1.length !== keys2.length)
            return false;
        const keys = [...new Set(keys1.concat(keys2))];
        return keys.every((key) => key in val1 && key in val2 && Differ.equal(val1[key], val2[key]));
    }
    static #getDiff(val1, val2, parent) {
        if (!Differ.#type().object(val1) || !Differ.#type().object(val2))
            return [];
        const keys = [...new Set(Object.keys(val1).concat(Object.keys(val2)))];
        const diff = [];
        for (const key of keys) {
            const parentKey = (parent ? `${parent}.` : '') + key;
            if (!(key in val1) || !(key in val2)) {
                diff.push(parentKey);
                continue;
            }
            const obj1 = val1[key];
            const obj2 = val2[key];
            if (Differ.#type().object(obj1) && Differ.#type().object(obj2))
                diff.push(...Differ.#getDiff(obj1, obj2, parentKey));
            else if (!Differ.equal(obj1, obj2))
                diff.push(parentKey);
        }
        return diff;
    }
    static diff(val1, val2) {
        return Differ.#getDiff(val1, val2);
    }
    static from(keys) {
        const deepMerge = (from, to) => Object.keys(from)
            .reduce((merged, key) => {
            const obj = from[key];
            merged[key] = Differ.#type().object(obj) ? deepMerge(obj, merged[key] ?? {}) : obj;
            return merged;
        }, { ...to });
        const formObject = (key) => key.split('.').reverse().reduce((acc, k) => ({ [k]: acc ?? true }), null);
        return keys.map(formObject).reduce(deepMerge, {});
    }
}
exports.Differ = Differ;
//# sourceMappingURL=differ.js.map